# frozen_string_literal: true

require 'interactor'
require_relative 'interactor/failure'

module Overture
  # This is a subclass of [`::Interactor`](https://github.com/collectiveidea/interactor)
  # with some methods added for comodity.
  class Interactor
    include ::Interactor

    class << self
      # Mark context keys as required.
      #
      # If any of the specified keys are sent as `nil`, it will
      # fail with a message like `'<key> is required'`.
      #
      # Possible options are:
      #
      # - `allow_empty` Allows empty values such as `[]`, `''`
      #
      # Example:
      #
      # class MyInteractor < Overture::Interactor
      #   required :first_name, :last_name, allow_empty: true
      # end
      def required(*input)
        before do
          opts = input.last.is_a?(Hash) ? input.pop : {}
          allow_empty = opts[:allow_empty]

          input.each do |attr|
            value = context.public_send(attr)
            value = value.strip if value.respond_to?(:strip)
            empty = value.respond_to?(:empty?) ? value.empty? : false
            valid = !value.nil? && (allow_empty ? true : !empty)
            fail!("#{attr} is required") unless valid
          end
        end
      end
    end

    # Fails context and sets `context.error` attribute to the passed message.
    #
    # An optional hash can be passed that will get merged when failing.
    def fail!(message, meta = {})
      context.fail!(meta.merge(error: message))
    end

    # Checks for presence and non-emptiness
    def blank?(value)
      return true if value.nil?

      value = value.strip if value.respond_to?(:strip)
      value.empty?
    end

    # Slice the passed keys from context if they're present.
    #
    # When `include_nils` is false, `nil` values are ignored.
    def context_slice(*keys, include_nils: false)
      keys.each_with_object({}) do |key, memo|
        next unless context_key?(key)

        memo[key] = context[key] if !context[key].nil? || include_nils
      end
    end

    def context_key?(key)
      context.to_h.key?(key.to_sym)
    end
  end
end
